﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using UserService.DataModels;

namespace UserService.Repository
{
    public class OrganizationUnitRepository : IOrganizationUnitRepository
    {
        public async Task<IReadOnlyList<OrganizationUnit>> GetAll(CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            return await db.OrganizationUnits.ToListAsync(token);
        }

        /// <inheritdoc />
        public async Task<OrganizationUnit?> Get(Func<OrganizationUnit, bool> predicate,
            CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            return db.OrganizationUnits.Where(predicate).FirstOrDefault();
        }

        public async Task Add(OrganizationUnit entity, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            await db.OrganizationUnits.AddAsync(@entity, token);
            await db.SaveChangesAsync(token);
        }

        public async Task Update(OrganizationUnit entity, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            db.OrganizationUnits.Update(entity);
            await db.SaveChangesAsync(token);
        }

        public async Task Delete(OrganizationUnit entity, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            db.OrganizationUnits.Remove(entity);
            await db.SaveChangesAsync(token);
        }
    }

    
    public class SecurityGroupsRepository : ISecurityGroupsRepository
    {
        public async Task<IReadOnlyList<SecurityGroup>> GetAll(CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            return await db.SecurityGroups.ToListAsync(token);
        }

        /// <inheritdoc />
        public async Task<SecurityGroup?> Get(Func<SecurityGroup, bool> predicate, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            return db.SecurityGroups.Where(predicate).FirstOrDefault();
        }

        public async Task Add(SecurityGroup entity, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            await db.SecurityGroups.AddAsync(@entity, token);
            await db.SaveChangesAsync(token);
        }

        public async Task Update(SecurityGroup entity, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            db.SecurityGroups.Update(entity);
            await db.SaveChangesAsync(token);
        }

        public async Task Delete(SecurityGroup entity, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            db.SecurityGroups.Remove(entity);
            await db.SaveChangesAsync(token);
        }
    }


    public class UsersRepository : IUsersRepository
    {
        public async Task<IReadOnlyList<User>> GetAll(CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            return await db.Users.ToListAsync(token);
        }

        /// <inheritdoc />
        public async Task<User?> Get(Func<User, bool> predicate, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            return db.Users.Where(predicate).FirstOrDefault();
        }

        public async Task Add(User entity, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            await db.Users.AddAsync(@entity, token);
            await db.SaveChangesAsync(token);
        }

        public async Task Update(User entity, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            db.Users.Update(entity);
            await db.SaveChangesAsync(token);
        }

        public async Task Delete(User entity, CancellationToken token = default)
        {
            await using var db = new GroupsContext();
            db.Users.Remove(entity);
            await db.SaveChangesAsync(token);
        }
    }
}