﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using UserService.DataModels;

namespace UserService.Repository
{
    public interface IRepository<T> where T : Node
    {
        Task<IReadOnlyList<T>> GetAll(CancellationToken token = default);
        Task<T?> Get(Func<T, bool> predicate, CancellationToken token = default);
        Task Add(T entity, CancellationToken token = default);
        Task Update(T entity, CancellationToken token = default);
        Task Delete(T entity, CancellationToken token = default);
    }

    public interface IOrganizationUnitRepository : IRepository<OrganizationUnit>
    {

    }

    public interface ISecurityGroupsRepository : IRepository<SecurityGroup>
    {

    }

    public interface IUsersRepository : IRepository<User>
    {

    }
}